
#include <cairo/cairo.h>
#include <gtk/gtk.h>

cairo_surface_t *pattern_surface;
cairo_pattern_t *pattern;

static void test(cairo_t *cr)
{
	cairo_matrix_t matrix;
	cairo_matrix_init_translate(&matrix, 14.1, 0);
	cairo_pattern_set_matrix(pattern, &matrix);

	cairo_set_source(cr, pattern);

	for (int i = 0; i < 8; i++){
		cairo_rectangle(cr, i * 3.5, i * 32, 256, 32);
		cairo_fill(cr);
	}
}

static gboolean on_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer data)
{
  cairo_t *cr;
  cr = gdk_cairo_create(widget->window);
	test(cr);
	cairo_destroy(cr);
  return FALSE;
}


int main (int argc, char *argv[])
{
  GtkWidget *window;
  GtkWidget *darea;

  gtk_init(&argc, &argv);

	pattern_surface = cairo_image_surface_create_from_png("pattern.png");
	pattern = cairo_pattern_create_for_surface(pattern_surface);
	cairo_pattern_set_extend(pattern, CAIRO_EXTEND_REPEAT);

	cairo_t *cr;
	cairo_surface_t *surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 256, 256);
	cr = cairo_create(surface);
	test(cr);
	cairo_destroy(cr);
	cairo_surface_write_to_png(surface, "test.png");
	cairo_surface_destroy(surface);

  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

  darea = gtk_drawing_area_new();
  gtk_container_add(GTK_CONTAINER(window), darea);

  g_signal_connect(darea, "expose-event", G_CALLBACK(on_expose_event), NULL);
  g_signal_connect(window, "destroy", G_CALLBACK(gtk_main_quit), NULL);

  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
  gtk_window_set_default_size(GTK_WINDOW(window), 256, 256);

  gtk_widget_show_all(window);
  gtk_main();

	cairo_pattern_destroy(pattern);
	cairo_surface_destroy(pattern_surface);

  return 0;
}


